@@ -1,4 +1,4 @@
-/* Copyright 2011--2016 The Tor Project
+/* Copyright 2011--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.onionoo.server;
@@ -10,6 +10,7 @@ import org.torproject.onionoo.docs.SummaryDocument;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -133,11 +134,9 @@ public class RequestHandler {
     this.family = family;
   }
 
-  private Map<String, SummaryDocument> filteredRelays =
-      new HashMap<String, SummaryDocument>();
+  private Map<String, SummaryDocument> filteredRelays = new HashMap<>();
 
-  private Map<String, SummaryDocument> filteredBridges =
-      new HashMap<String, SummaryDocument>();
+  private Map<String, SummaryDocument> filteredBridges = new HashMap<>();
 
   /** Handles this request by filtering by all given parameters and then
    * possibly ordering, offsetting, and limiting results. */
@@ -190,7 +189,7 @@ public class RequestHandler {
       return;
     }
     boolean runningRequested = this.running.equals("true");
-    Set<String> removeRelays = new HashSet<String>();
+    Set<String> removeRelays = new HashSet<>();
     for (Map.Entry<String, SummaryDocument> e
         : filteredRelays.entrySet()) {
       if (e.getValue().isRunning() != runningRequested) {
@@ -200,7 +199,7 @@ public class RequestHandler {
     for (String fingerprint : removeRelays) {
       this.filteredRelays.remove(fingerprint);
     }
-    Set<String> removeBridges = new HashSet<String>();
+    Set<String> removeBridges = new HashSet<>();
     for (Map.Entry<String, SummaryDocument> e
         : filteredBridges.entrySet()) {
       if (e.getValue().isRunning() != runningRequested) {
@@ -223,7 +222,7 @@ public class RequestHandler {
   }
 
   private void filterBySearchTerm(String searchTerm) {
-    Set<String> removeRelays = new HashSet<String>();
+    Set<String> removeRelays = new HashSet<>();
     for (Map.Entry<String, SummaryDocument> e
         : filteredRelays.entrySet()) {
       String fingerprint = e.getKey();
@@ -276,7 +275,7 @@ public class RequestHandler {
     for (String fingerprint : removeRelays) {
       this.filteredRelays.remove(fingerprint);
     }
-    Set<String> removeBridges = new HashSet<String>();
+    Set<String> removeBridges = new HashSet<>();
     for (Map.Entry<String, SummaryDocument> e :
         filteredBridges.entrySet()) {
       String hashedFingerprint = e.getKey();
@@ -356,7 +355,7 @@ public class RequestHandler {
     } else {
       Set<String> relaysWithCountryCode =
           this.nodeIndex.getRelaysByCountryCode().get(countryCode);
-      Set<String> removeRelays = new HashSet<String>();
+      Set<String> removeRelays = new HashSet<>();
       for (String fingerprint : this.filteredRelays.keySet()) {
         if (!relaysWithCountryCode.contains(fingerprint)) {
           removeRelays.add(fingerprint);
@@ -383,7 +382,7 @@ public class RequestHandler {
     } else {
       Set<String> relaysWithAsNumber =
           this.nodeIndex.getRelaysByAsNumber().get(asNumber);
-      Set<String> removeRelays = new HashSet<String>();
+      Set<String> removeRelays = new HashSet<>();
       for (String fingerprint : this.filteredRelays.keySet()) {
         if (!relaysWithAsNumber.contains(fingerprint)) {
           removeRelays.add(fingerprint);
@@ -407,7 +406,7 @@ public class RequestHandler {
     } else {
       Set<String> relaysWithFlag = this.nodeIndex.getRelaysByFlag().get(
           flag);
-      Set<String> removeRelays = new HashSet<String>();
+      Set<String> removeRelays = new HashSet<>();
       for (String fingerprint : this.filteredRelays.keySet()) {
         if (!relaysWithFlag.contains(fingerprint)) {
           removeRelays.add(fingerprint);
@@ -422,7 +421,7 @@ public class RequestHandler {
     } else {
       Set<String> bridgesWithFlag = this.nodeIndex.getBridgesByFlag().get(
           flag);
-      Set<String> removeBridges = new HashSet<String>();
+      Set<String> removeBridges = new HashSet<>();
       for (String fingerprint : this.filteredBridges.keySet()) {
         if (!bridgesWithFlag.contains(fingerprint)) {
           removeBridges.add(fingerprint);
@@ -459,7 +458,7 @@ public class RequestHandler {
   private void filterNodesByDays(
       Map<String, SummaryDocument> filteredNodes,
       SortedMap<Integer, Set<String>> nodesByDays, int[] days) {
-    Set<String> removeNodes = new HashSet<String>();
+    Set<String> removeNodes = new HashSet<>();
     for (Set<String> nodes : nodesByDays.headMap(days[0]).values()) {
       removeNodes.addAll(nodes);
     }
@@ -479,7 +478,7 @@ public class RequestHandler {
       /* Not filtering by contact information. */
       return;
     }
-    Set<String> removeRelays = new HashSet<String>();
+    Set<String> removeRelays = new HashSet<>();
     for (Map.Entry<String, Set<String>> e :
         this.nodeIndex.getRelaysByContact().entrySet()) {
       String contact = e.getKey();
@@ -502,8 +501,7 @@ public class RequestHandler {
       /* Not filtering by relay family. */
       return;
     }
-    Set<String> removeRelays = new HashSet<String>(
-        this.filteredRelays.keySet());
+    Set<String> removeRelays = new HashSet<>(this.filteredRelays.keySet());
     removeRelays.remove(this.family);
     if (this.nodeIndex.getRelaysByFamily().containsKey(this.family)) {
       removeRelays.removeAll(this.nodeIndex.getRelaysByFamily()
@@ -516,34 +514,38 @@ public class RequestHandler {
   }
 
   private void order() {
-    if (this.order != null && this.order.length == 1) {
-      List<String> orderBy = new ArrayList<String>(
-          this.nodeIndex.getRelaysByConsensusWeight());
-      if (this.order[0].startsWith("-")) {
-        Collections.reverse(orderBy);
+    List<SummaryDocument> uniqueRelays = new ArrayList<>();
+    List<SummaryDocument> uniqueBridges = new ArrayList<>();
+    for (SummaryDocument relay : this.filteredRelays.values()) {
+      if (!uniqueRelays.contains(relay)) {
+        uniqueRelays.add(relay);
       }
-      for (String relay : orderBy) {
-        if (this.filteredRelays.containsKey(relay)
-            && !this.orderedRelays.contains(filteredRelays.get(relay))) {
-          this.orderedRelays.add(this.filteredRelays.remove(relay));
-        }
-      }
-      for (String relay : this.filteredRelays.keySet()) {
-        if (!this.orderedRelays.contains(this.filteredRelays.get(relay))) {
-          this.orderedRelays.add(this.filteredRelays.remove(relay));
-        }
-      }
-      Set<SummaryDocument> uniqueBridges = new HashSet<SummaryDocument>(
-          this.filteredBridges.values());
-      this.orderedBridges.addAll(uniqueBridges);
-    } else {
-      Set<SummaryDocument> uniqueRelays = new HashSet<SummaryDocument>(
-          this.filteredRelays.values());
-      this.orderedRelays.addAll(uniqueRelays);
-      Set<SummaryDocument> uniqueBridges = new HashSet<SummaryDocument>(
-          this.filteredBridges.values());
-      this.orderedBridges.addAll(uniqueBridges);
     }
+    for (SummaryDocument bridge : this.filteredBridges.values()) {
+      if (!uniqueBridges.contains(bridge)) {
+        uniqueBridges.add(bridge);
+      }
+    }
+    if (this.order != null) {
+      Comparator<SummaryDocument> comparator
+          = new SummaryDocumentComparator(this.order);
+      Collections.sort(uniqueRelays, comparator);
+      Collections.sort(uniqueBridges, comparator);
+    }
+    this.orderedRelays.addAll(uniqueRelays);
+    this.orderedBridges.addAll(uniqueBridges);
+  }
+
+  private int relaysSkipped = 0;
+
+  public int getRelaysSkipped() {
+    return this.relaysSkipped;
+  }
+
+  private int bridgesSkipped = 0;
+
+  public int getBridgesSkipped() {
+    return this.bridgesSkipped;
   }
 
   private void offset() {
@@ -557,12 +559,26 @@ public class RequestHandler {
         || !this.orderedBridges.isEmpty())) {
       if (!this.orderedRelays.isEmpty()) {
         this.orderedRelays.remove(0);
+        this.relaysSkipped++;
       } else {
         this.orderedBridges.remove(0);
+        this.bridgesSkipped++;
       }
     }
   }
 
+  private int relaysTruncated = 0;
+
+  public int getRelaysTruncated() {
+    return this.relaysTruncated;
+  }
+
+  private int bridgesTruncated = 0;
+
+  public int getBridgesTruncated() {
+    return this.bridgesTruncated;
+  }
+
   private void limit() {
     if (this.limit == null) {
       /* Not limiting number of results. */
@@ -572,23 +588,23 @@ public class RequestHandler {
     while (!this.orderedRelays.isEmpty()
         && limitValue < this.orderedRelays.size()) {
       this.orderedRelays.remove(this.orderedRelays.size() - 1);
+      this.relaysTruncated++;
     }
     limitValue -= this.orderedRelays.size();
     while (!this.orderedBridges.isEmpty()
         && limitValue < this.orderedBridges.size()) {
       this.orderedBridges.remove(this.orderedBridges.size() - 1);
+      this.bridgesTruncated++;
     }
   }
 
-  private List<SummaryDocument> orderedRelays =
-      new ArrayList<SummaryDocument>();
+  private List<SummaryDocument> orderedRelays = new ArrayList<>();
 
   public List<SummaryDocument> getOrderedRelays() {
     return this.orderedRelays;
   }
 
-  private List<SummaryDocument> orderedBridges =
-      new ArrayList<SummaryDocument>();
+  private List<SummaryDocument> orderedBridges = new ArrayList<>();
 
   public List<SummaryDocument> getOrderedBridges() {
     return this.orderedBridges;
