@@ -1,4 +1,4 @@
-/* Copyright 2011--2016 The Tor Project
+/* Copyright 2011--2017 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.onionoo.server;
@@ -66,13 +66,13 @@ public class ResourceServlet extends HttpServlet {
 
   private static final long CACHE_INTERVAL = 5L * 60L * 1000L;
 
-  private static Set<String> knownParameters = new HashSet<String>(
+  private static Set<String> knownParameters = new HashSet<>(
       Arrays.asList(("type,running,search,lookup,fingerprint,country,as,"
           + "flag,first_seen_days,last_seen_days,contact,order,limit,"
           + "offset,fields,family").split(",")));
 
   private static Set<String> illegalSearchQualifiers =
-      new HashSet<String>(Arrays.asList(("search,fingerprint,order,limit,"
+      new HashSet<>(Arrays.asList(("search,fingerprint,order,limit,"
           + "offset,fields").split(",")));
 
   /** Handles the HTTP GET request in the wrapped <code>request</code> by
@@ -124,7 +124,7 @@ public class ResourceServlet extends HttpServlet {
 
     /* Extract parameters either from the old-style URI or from request
      * parameters. */
-    Map<String, String> parameterMap = new HashMap<String, String>();
+    Map<String, String> parameterMap = new HashMap<>();
     for (Object parameterKey : request.getParameterMap().keySet()) {
       String[] parameterValues =
           request.getParameterValues((String) parameterKey);
@@ -148,7 +148,7 @@ public class ResourceServlet extends HttpServlet {
         response.sendError(HttpServletResponse.SC_BAD_REQUEST);
         return;
       }
-      List<String> unqualifiedSearchTerms = new ArrayList<String>();
+      List<String> unqualifiedSearchTerms = new ArrayList<>();
       for (String searchTerm : searchTerms) {
         if (searchTerm.contains(":") && !searchTerm.startsWith("[")) {
           String[] parts = searchTerm.split(":", 2);
@@ -267,16 +267,12 @@ public class ResourceServlet extends HttpServlet {
       rh.setContact(contactParts);
     }
     if (parameterMap.containsKey("order")) {
-      String orderParameter = parameterMap.get("order").toLowerCase();
-      String orderByField = orderParameter;
-      if (orderByField.startsWith("-")) {
-        orderByField = orderByField.substring(1);
-      }
-      if (!orderByField.equals("consensus_weight")) {
+      String[] order = this.parseOrderParameter(parameterMap.get("order"));
+      if (order == null) {
         response.sendError(HttpServletResponse.SC_BAD_REQUEST);
         return;
       }
-      rh.setOrder(new String[] { orderParameter });
+      rh.setOrder(order);
     }
     if (parameterMap.containsKey("offset")) {
       String offsetParameter = parameterMap.get("offset");
@@ -325,6 +321,10 @@ public class ResourceServlet extends HttpServlet {
     rb.setBridgesPublishedString(rh.getBridgesPublishedString());
     rb.setOrderedRelays(rh.getOrderedRelays());
     rb.setOrderedBridges(rh.getOrderedBridges());
+    rb.setRelaysSkipped(rh.getRelaysSkipped());
+    rb.setBridgesSkipped(rh.getBridgesSkipped());
+    rb.setRelaysTruncated(rh.getRelaysTruncated());
+    rb.setBridgesTruncated(rh.getBridgesTruncated());
     String[] fields = null;
     if (parameterMap.containsKey("fields")) {
       fields = this.parseFieldsParameter(parameterMap.get("fields"));
@@ -362,7 +362,7 @@ public class ResourceServlet extends HttpServlet {
 
   private static Pattern searchQueryStringPattern =
       Pattern.compile("(?:.*[\\?&])*?" // lazily skip other parameters
-          + "search=([0-9a-zA-Z+/\\.: \\$\\[\\]%]+)" // capture parameter
+          + "search=([\\p{Graph} &&[^&]]+)" // capture parameter
           + "(?:&.*)*"); // skip remaining parameters
 
   private static Pattern searchParameterPattern =
@@ -370,7 +370,7 @@ public class ResourceServlet extends HttpServlet {
       + "^[0-9a-zA-Z+/]{1,27}$|" /* Base64 fingerprint. */
       + "^[0-9a-zA-Z\\.]{1,19}$|" /* Nickname or IPv4 address. */
       + "^\\[[0-9a-fA-F:\\.]{1,39}\\]?$|" /* IPv6 address. */
-      + "^[a-zA-Z_]+:[0-9a-zA-Z_,-]+$" /* Qualified search term. */);
+      + "^[a-zA-Z_]+:\\p{Graph}+$" /* Qualified search term. */);
 
   protected static String[] parseSearchParameters(String queryString) {
     Matcher searchQueryStringMatcher = searchQueryStringPattern.matcher(
@@ -483,6 +483,36 @@ public class ResourceServlet extends HttpServlet {
     return parameter.split(" ");
   }
 
+  private static Pattern orderParameterPattern =
+      Pattern.compile("^[0-9a-zA-Z_,-]*$");
+
+  private static HashSet<String> knownOrderParameters = new HashSet<>(
+      Arrays.asList(new String[] { OrderParameterValues.CONSENSUS_WEIGHT_ASC,
+          OrderParameterValues.CONSENSUS_WEIGHT_DES,
+          OrderParameterValues.FIRST_SEEN_ASC,
+          OrderParameterValues.FIRST_SEEN_DES }));
+
+  private String[] parseOrderParameter(String parameter) {
+    if (!orderParameterPattern.matcher(parameter).matches()) {
+      /* Orders contain illegal character(s). */
+      return null;
+    }
+    String[] orderParameters = parameter.toLowerCase().split(",");
+    Set<String> seenOrderParameters = new HashSet<>();
+    for (String orderParameter : orderParameters) {
+      if (!knownOrderParameters.contains(orderParameter)) {
+        /* Unknown order parameter. */
+        return null;
+      }
+      if (!seenOrderParameters.add(orderParameter.startsWith("-")
+          ? orderParameter.substring(1) : orderParameter)) {
+        /* Duplicate parameter. */
+        return null;
+      }
+    }
+    return orderParameters;
+  }
+
   private static Pattern fieldsParameterPattern =
       Pattern.compile("^[0-9a-zA-Z_,]*$");
 
